###########################################################################/**
# @RdocClass SocketSemaphore
#
# @title "The SocketSemaphore class"
#
# \description{
#  @classhierarchy
#
#  A SocketSemaphore is a class for acquiring a semphore from a server.
# }
# 
# @synopsis
#
# \arguments{
#   \item{host, port}{The hostname and port number of the server.}
#   \item{...}{Not used.}
# }
#
# \section{Fields and Methods}{
#  @allmethods "public"
# }
# 
# \examples{\dontrun{
#   @include "../incl/SocketSemaphoreServer.Rex"
#
#   @include "../incl/SocketSemaphore.Rex"
# }}
#
# @author
#
# @keyword file
#*/########################################################################### 
setConstructorS3("SocketSemaphore", function(host="127.0.0.1", port=7777, ...) {
  # Argument 'host':
  host <- Arguments$getCharacter(host);

  # Argument 'port':
  port <- Arguments$getInteger(port, range=c(0,65535));

  extend(AbstractSemaphore(), "SocketSemaphore",
    .host = host,
    .port = port,
    .con = NULL
  );
})

setMethodS3("finalize", "SocketSemaphore", function(this, ...) {
  release(this);
})


setMethodS3("isAcquired", "SocketSemaphore", function(this, ..., verbose=FALSE) {
  # Argument 'verbose':
  verbose <- Arguments$getVerbose(verbose);

  verbose && enter(verbose, "Check if connected");
  con <- this$.con;
  res <- (!is.null(con));
  verbose && str(verbose, res);
  verbose && exit(verbose);

  res;
})


setMethodS3("release", "SocketSemaphore", function(this, ..., verbose=FALSE) {
  con <- this$.con;
  if (!is.null(con)) {
    this$.con <- NULL;
    close(con);
    con <- NULL;
  }
  invisible(isAcquired(this));
})


setMethodS3("tryAcquire", "SocketSemaphore", function(this, timeout=0.5, ..., verbose=FALSE) {
  # Argument 'verbose':
  verbose <- Arguments$getVerbose(verbose);
 

  if (isAcquired(this)) {
    throw("Already acquired.");
  }

  oopts <- getOption("timeout");
  on.exit({
    options(timeout=oopts);
  });
  options(timeout=timeout);

  host <- this$.host;
  port <- this$.port;
  
  verbose && enter(verbose, "Trying to connect to server");
  con <- NULL;
  tryCatch({
    suppressWarnings({
      con <- socketConnection(host=host, port=port, 
                            server=FALSE, blocking=FALSE, open="a+");
    });
    this$.con <- con;
    verbose && exit(verbose);
  }, error = function(ex) {
    verbose && exit(verbose, suffix="...failed.");
  });

  invisible(isAcquired(this));
})


############################################################################
# HISTORY:
# 2009-05-29
# o Created.
############################################################################ 
